#
# spec file for package katacontainers-image-initrd
#
# Copyright (c) 2020 SUSE LLC
#
# All modifications and additions to the file contributed by third parties
# remain the property of their copyright owners, unless otherwise agreed
# upon. The license for this file, and modifications and additions to the
# file, is the same license as for the pristine package itself (unless the
# license for the pristine package is not an Open Source License, in which
# case the license is the MIT License). An "Open Source License" is a
# license that conforms to the Open Source Definition (Version 1.9)
# published by the Open Source Initiative.

# Please submit bugfixes or comments via https://bugs.opensuse.org/
#


#
%define kata_agent_project    github.com/kata-containers/agent

# Platforms that supports kvm specific kernels
%ifarch x86_64
%define kernel_flavor         kvmsmall
%else
%define kernel_flavor         default
%endif

%ifarch x86_64
%define kernel_basename       vmlinuz
%endif

%ifarch ppc64le
%define kernel_basename       vmlinux
%endif

%ifarch aarch64
%define kernel_basename       Image
%define kernel_compress       1
%endif

%ifarch s390x
%define kernel_basename       image
%endif

Name:           katacontainers-image-initrd
Version:        1.10.0
Release:        lp152.1.39
Summary:        Kata Containers image (initrd) and kernel
License:        Apache-2.0
Group:          System/Management
URL:            https://github.com/kata-containers/osbuilder
Source0:        osbuilder-%{version}.tar.xz
Source1:        agent-%{version}.tar.xz
# PATCH-FIX-UPSTREAM agent_memory-hotplug-probe.patch https://github.com/kata-containers/agent/issues/712
Patch0:         agent_memory-hotplug-probe.patch
ExclusiveArch:  x86_64 aarch64 ppc64le s390x
BuildRequires:  dracut
BuildRequires:  fdupes
BuildRequires:  kernel-%{kernel_flavor}
BuildRequires:  golang(API) >= 1.12
Provides:       katacontainers-image

%description
Kata Containers image (in initrd format) and kernel used for the ephemeral VMs
used to run containers.

%prep
%setup -q -n osbuilder-%{version} -b1
pushd ../agent-%{version}
%patch0 -p1
popd

%build
export GOPATH=$HOME/go
rm -rf $HOME/go/src/%{kata_agent_project}
mkdir -pv $HOME/go/src/%{kata_agent_project}
cp -avr ../agent-%{version}/* $HOME/go/src/%{kata_agent_project}

kata_kmodules=(9p 9pnet 9pnet_virtio)

%if 0%{?suse_version} <= 1500 || %{kernel_flavor} != "kvmsmall"
kata_kmodules+=( \
virtio \
virtio_pci \
virtio_balloon \
virtio_blk \
virtio_console \
virtio_crypto \
virtio-gpu \
virtio_input \
virtio_net \
virtio-rng \
virtio_scsi \
vmw_vsock_virtio_transport \
vmw_vsock_virtio_transport_common \
)
%endif

# Minimal set of kernel modules to allow starting containers
echo "drivers=\"${kata_kmodules[@]}\"" > dracut/dracut.conf.d/10-drivers.conf
kversion="$(find /boot -name '%{kernel_basename}-*%{kernel_flavor}' | sed 's,/boot/%{kernel_basename}-,,' | sort | tail -n 1)"
echo "Found kernel version: ${kversion}"
echo "${kversion}" > %{_builddir}/kversion

make \
    BUILD_METHOD=dracut \
    DRACUT_KVERSION=${kversion} \
    clean initrd

%check
# (tested manually)

%install
read kversion < %{_builddir}/kversion
# image
install -m 0644 -D kata-containers-initrd.img %{buildroot}%{_datarootdir}/kata-containers/kata-containers-initrd-${kversion}.img
ln -sf kata-containers-initrd-${kversion}.img %{buildroot}%{_datarootdir}/kata-containers/kata-containers-initrd.img

#kernel
vmKernelDest="%{buildroot}%{_datarootdir}/kata-containers/%{kernel_basename}-${kversion}"
%if 0%{?kernel_compress}
gzip -c /boot/%{kernel_basename}-${kversion} > ${vmKernelDest}.gz
%else
install -m 0644 -D /boot/%{kernel_basename}-${kversion} ${vmKernelDest}
%endif
ln -sf "$(basename "${vmKernelDest}%{?kernel_compress:.gz}")" %{buildroot}%{_datarootdir}/kata-containers/vmlinuz

%fdupes %{buildroot}/%{_prefix}

%files
%defattr(-,root,root,-)
%dir %{_datarootdir}/kata-containers
%{_datarootdir}/kata-containers/kata-containers-initrd.img
%{_datarootdir}/kata-containers/kata-containers-initrd-*-%{kernel_flavor}.img
%{_datarootdir}/kata-containers/vmlinuz
%{_datarootdir}/kata-containers/%{kernel_basename}-*-%{kernel_flavor}%{?kernel_compress:.gz}
%license LICENSE

%changelog
* Mon Jan 27 2020 Ralf Haferkamp <rhafer@suse.com>
- kata-agent: add patch agent_memory-hotplug-probe.patch to address
  memory hotplug issue when running with SUSE kernel
  (https://github.com/kata-containers/agent/issues/712)
* Mon Jan 20 2020 Ralf Haferkamp <rhafer@suse.com>
- Update to 1.10.0:
  agent:
  - client.go: HybridVSockDialer: Check return size n of
    unix.Recvfrom
  - client.go: HybridVSockDialer: Change Read EOT to recv peek
  - agent: get current cpuset from /sys/devices/system/cpu/online
  - config: add option to control hotplug timeout of block
    devices
  - agent: firecracker: support debugging console
  - agent: fix pause bin on musl
  - docs: Add enable services and installation steps for TRACING.md
  - docs: Add missing steps at TRACING.md
  - tracing: Wrapper for tracing functions
  - tracing: Generate an alias for opentracing.Span
  - network: ensure parent directories exist
  - Revert "client: remove the parameter of 'enableYamux'"
  - client: remove the parameter of 'enableYamux'
  - vendor: update libcontainer to 1.0.0-rc9
  osbuilder:
  - make: use `cd` instead of '--directory' option of `cpio`
* Fri Dec 13 2019 Ralf Haferkamp <rhafer@suse.com>
- Update to 1.9.3 tarballs, this is just a version bump without
  any code changes.
* Wed Dec  4 2019 Ralf Haferkamp <rhafer@suse.com>
- Update to 1.9.2 tarballs, this is just a version bump without
  any code changes.
- Add more missing virtio kernel modules to initrd to fix
  issues with SLE15-SP1
* Thu Nov 14 2019 Ralf Haferkamp <rhafer@suse.com>
- Update to 1.9.1 tarballs, this is just a version bump without
  any code changes.
* Tue Nov  5 2019 Ralf Haferkamp <rhafer@suse.com>
- Different to Tumbleweed the kvmsmall kernel flavor on Leap
  (and SLE) 15.1 doesn't statically include most of the virtio
  features. So add them to the initrd on these distros.
* Thu Oct 24 2019 Ralf Haferkamp <rhafer@suse.com>
- Update to 1.9.0 tarballs, this is just a version bump without
  any code changes.
* Mon Oct 21 2019 Ralf Haferkamp <rhafer@suse.com>
- Update to 1.9.0~rc0:
  agent:
  - firecracker: send logs to a vsock port
  - network: Setup DNS for sandbox
  - protocols/client: support hybrid vsocks
  - Fix incorrect script reference in TRACING.md
  osbuilder:
  - osbuilder: Add USE_PODMAN as an alternate for USE_DOCKER
  - README: Update how to use DEBUG variable
  - Fix yq
  - initrd_builder: Create empty resolv.conf in rootfs
  - rootfs: update fedora to 29 for arm64.
- Fix config for set-version source-service to set version based
  on the osbuilder filename. The previously used
  "katacontainers-image-initrd" caused the service to fail.
* Wed Sep 25 2019 Marco Vedovati <mvedovati@suse.com>
- Update package version to 1.9.0~alpha2:
  agent 1.9.0~alpha2:
  * mount: ensure local directory storage types have the correct permissions
  * device: fix the issue of failed waiting on device appeared in /dev
  * network: While updating routes, do not delete routes with proto "kernel"
  * agent: add support for loading kernel modules
  * revert: agent: sandbox_pause should get arguments from proc
  * agent: sandbox_pause should get arguments from proc
  * s390x: add virtio-blk-ccw support
  * agent: make NoPivotRoot config depend on / fs type
  * make: install depends on $(TARGET)
  * device: Allow uevent handler to be stopped
  * mount: Improve error message
  * memory: Add extra check for memory file
* Tue Sep  3 2019 Marco Vedovati <mvedovati@suse.com>
- Add support for non x86_64 archs, by using the default kernel flavor instead
  of kvmsmall
- Require golang >= 1.12
- Update package summary and description
- Set the list of exclusive archs (x86_64, aarch64, ppc64le, s390x)
* Fri Aug 30 2019 Marco Vedovati <mvedovati@suse.com>
- Vendor in vmlinuz from kernel-kvmsmall to avoid dependency on that package
  and the side effect linked to installing a VM kernel on the host
* Thu Aug  8 2019 Marco Vedovati <mvedovati@suse.com>
- Have this package directly depend on kernel-kvmsmall.
* Mon Aug  5 2019 Marco Vedovati <mvedovati@suse.com>
- Include kernel version in initrd image name, and symlink it to the generic
  image name. This allows the katacontainers package to retrieve the correct
  kernel version to use
- Rename kata-osbuilder tarball to osbuilder
* Wed Jul 31 2019 Marco Vedovati <mvedovati@suse.com>
- Initial package creation version 1.9.0~alpha0
